#!/bin/sh -e
# WARNING: it may fail if ~/.sqliterc exists!
# Convert Pyshop working directory to GoCheese friendly one.
# Script expects to be run in packages repository (a/, b/, ...)
# with pyshop.db database file. It will create packages/ directory
# with all Pyshop-downloaded/uploaded files, SHA256 checksums,
# private package marks. Also it will call GoCheese's /simple/ API
# for forcing metainformation update (necessary for storing SHA256
# checksums missing in Pyshop). No metadata conversion is performed.

pkgname() {
    perl -ne "s/[-_.]+/-/g ; print lc"
}

########################################################################
# Copy all already existing files
########################################################################
ctr=0
echo "
SELECT package.name, release_file.filename, release_file.created_at
FROM release_file
JOIN release ON release.id = release_file.release_id
JOIN package ON release.package_id = package.id
ORDER BY package.name
" | sqlite3 -separator "	" pyshop.db | while read pkginfo ; do
    ctr=$(( $ctr + 1 ))
    [ $(( $ctr % 100 )) -ne 0 ] || echo $ctr $pkginfo
    pkg=$(echo "$pkginfo" | cut -f1 | pkgname)
    filename=$(echo "$pkginfo" | cut -f2)
    created=$(echo "$pkginfo" | cut -f3 | sed "s/ /T/")
    [ -n "$pkg" ]
    [ -n "$filename" ]
    src=$(echo $pkg | cut -c1)/$filename
    dst=packages/$pkg/$filename
    [ -r $src ] || continue
    [ -r $dst ] && continue || :
    mkdir -p packages/$pkg
    ln $src $dst
    touch -d "$created" $dst
done

########################################################################
# Mark all private packages
########################################################################
for pkg in $(echo "SELECT name FROM package WHERE local = true" | sqlite3 pyshop.db); do
    cd packages/$(echo $pkg | pkgname)
    for f in * ; do b2sum -l 256 -a blake2b < $f | perl -lane 'print $F[0]' | xxd -r -p > $f.blake2b_256 ; done
    touch .internal
    touch -r $f $f.blake2b_256
    cd ../..
done

########################################################################
# Force all metainformation update from upstream
########################################################################
cd packages
for pkg in * ; do
    # Assume running:
    # GOCHEESE_NO_SYNC=1 gocheese -bind "[::1]:8080" -refresh /simple/
    curl http://localhost:8080/simple/$pkg/ > /dev/null
done

########################################################################
# Assure ckecksums are good
########################################################################
gocheese -fsck packages
